package cn.estsh.i3plus.ext.mes.pcn.api.busi;

import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesWorkOrderContext;
import cn.estsh.i3plus.pojo.mes.bean.MesMouldMultiCavity;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import io.swagger.annotations.ApiOperation;

import java.util.List;
import java.util.Map;

/**
 * 获取生产过程上下文对象接口【BUSI】
 */
public interface IMesProductionDispatchContextStepService {

    @ApiOperation(value = "发送设备质量信息")
    void sendEquipQualityMessage(StationRequestBean reqBean, MesCellEquipContext cellEquipContext);

    @ApiOperation(value = "存储展示组件MODULE_CONTENT内容")
    Boolean saveModuleContentContext(StationRequestBean reqBean, List<List<StationKvBean>> dataList);

    @ApiOperation(value = "获取展示组件MODULE_CONTENT内容")
    List<List<StationKvBean>> getModuleContentContext(StationRequestBean reqBean);

    @ApiOperation(value = "存储上个扫描/读取信息")
    Boolean saveLastScanContext(StationRequestBean reqBean, List<StationKvBean> dataList);

    @ApiOperation(value = "获取上个扫描/读取信息")
    List<StationKvBean> getLastScanContext(StationRequestBean reqBean);

    @ApiOperation(value = "发送工步内容")
    void doSendStepContextMessage(StationRequestBean reqBean);

    @ApiOperation(value = "发送工步内容")
    void doSendStepContextMessage(StationRequestBean reqBean, String scanInfo, MesExtEnumUtil.CELL_MESSAGE_SOURCE source);

    @ApiOperation(value = "发送工步内容")
    void sendStepContextMessage(StationRequestBean reqBean, List<StationKvBean> resultList);

    @ApiOperation(value = "获取上下文加工结果")
    String getProductResultContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文加工结果", notes = "加工结果:成功/报废/可疑 【写入的值是变量类别代码】【报废/可疑: 在保存逻辑条码时有特殊业务处理】")
    Boolean saveProductResultContext(StationRequestBean reqBean, String productResult);

    @ApiOperation(value = "删除上下文加工结果")
    void deleteProductResultContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文就绪信号")
    String getReadySignalContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文就绪信号", notes = "写1")
    Boolean saveReadySignalContext(StationRequestBean reqBean);

    @ApiOperation(value = "删除上下文就绪信号")
    void deleteReadySignalContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文头道模具号")
    MesEquipVariableCollectContext getFirstMouldNoContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文头道模具号是否存在")
    Boolean checkFirstMouldNoIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文头道模具号", notes = "[JSON]MesEquipVariableCollectContext::equipVariableValue")
    Boolean saveFirstMouldNoContext(StationRequestBean reqBean, MesEquipVariableCollectContext firstMouldNo);

    @ApiOperation(value = "删除上下文头道模具号")
    void deleteFirstMouldNoContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文一模多腔信息")
    List<MesMouldMultiCavity> getMouldMultiCavityContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文一模多腔信息是否存在")
    Boolean checkMouldMultiCavityIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文一模多腔信息", notes = "[JSON]List<MesMouldMultiCavity>")
    Boolean saveMouldMultiCavityContext(StationRequestBean reqBean, List<MesMouldMultiCavity> mouldMultiCavityList);

    @ApiOperation(value = "删除上下文一模多腔信息")
    void deleteMouldMultiCavityContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文零件数据信息")
    Map<String, MesPart> getPartDataContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文零件数据信息", notes = "[JSON]Map<String, MesPart>")
    Boolean savePartDataContext(StationRequestBean reqBean, Map<String, MesPart> partDataMap);

    @ApiOperation(value = "删除上下文零件数据信息")
    void deletePartDataContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文模具号")
    MesEquipVariableCollectContext getMouldNoContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文模具号是否存在")
    Boolean checkMouldNoIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文模具号", notes = "[JSON]MesEquipVariableCollectContext::equipVariableValue")
    Boolean saveMouldNoContext(StationRequestBean reqBean, MesEquipVariableCollectContext mouldNo);

    @ApiOperation(value = "删除上下文模具号")
    void deleteMouldNoContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文生产工单数据信息集合")
    List<MesWorkOrderContext> getWorkOrderDataContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文生产工单数据信息集合是否存在")
    Boolean checkWorkOrderDataIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文生产工单数据信息集合", notes = "[JSON]List<MesWorkOrderContext>")
    Boolean saveWorkOrderDataContext(StationRequestBean reqBean, List<MesWorkOrderContext> workOrderContextList);

    @ApiOperation(value = "删除上下文生产工单数据信息集合")
    void deleteWorkOrderDataContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文生产扫/读信息:主条码")
    List<MesEquipVariableCollectContext> getScanProductSnContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文扫/读信息:主条码是否存在")
    Boolean checkScanProductSnIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文扫/读信息:主条码", notes = "[JSON]List<MesEquipVariableCollectContext>")
    Boolean saveScanProductSnContext(StationRequestBean reqBean, List<MesEquipVariableCollectContext> productSnList);

    @ApiOperation(value = "删除上下文扫/读信息:装配件条码")
    void deleteScanProductSnContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文生产扫/读信息:装配件条码")
    List<MesEquipVariableCollectContext> getScanAssemblySnContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文扫/读信息:装配件条码是否存在")
    Boolean checkScanAssemblySnIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文扫/读信息:装配件条码", notes = "[JSON]List<MesEquipVariableCollectContext>")
    Boolean saveScanAssemblySnContext(StationRequestBean reqBean, List<MesEquipVariableCollectContext> productSnList);

    @ApiOperation(value = "删除上下文扫/读信息:装配件条码")
    void deleteScanAssemblySnContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文产出条码数据信息集合")
    List<MesProduceSn> getOutProduceSnDataContext(StationRequestBean reqBean);

    @ApiOperation(value = "验证上下文产出条码数据信息集合是否存在")
    Boolean checkOutProduceSnDataIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文产出条码数据信息集合", notes = "[JSON]List<MesProduceSn>")
    Boolean saveOutProduceSnDataContext(StationRequestBean reqBean, List<MesProduceSn> outProduceSnDataList);

    @ApiOperation(value = "删除上下文产出条码数据信息集合")
    void deleteOutProduceSnDataContext(StationRequestBean reqBean);

    @ApiOperation(value = "获取上下文装配件数据信息集合")
    List<MesProductionAssemblyContext> getAssemblyDataContext(StationRequestBean reqBean, MesWorkCenter workCenter);

    @ApiOperation(value = "验证上下文装配件数据信息集合是否存在")
    Boolean checkAssemblyDataIsExistContext(StationRequestBean reqBean);

    @ApiOperation(value = "保存上下文装配件数据信息集合", notes = "[JSON]List<MesProductionAssemblyContext>")
    Boolean saveAssemblyDataContext(StationRequestBean reqBean, List<MesProductionAssemblyContext> productionAssemblyContextList);

    @ApiOperation(value = "删除上下文装配件数据信息集合")
    void deleteAssemblyDataContext(StationRequestBean reqBean);

}
